@@ -12,15 +12,15 @@ cd /tmp/ || die "Can't chdir to /tmp/ ."
 
 if [ ! -r kernel-2.4.21-63.EL.src.rpm ]
 then
-    wget http://ftp.riken.jp/Linux/centos/3.9/updates/SRPMS/kernel-2.4.21-63.EL.src.rpm || die "Can't download source package."
+    wget http://vault.centos.org/3.9/updates/SRPMS/kernel-2.4.21-63.EL.src.rpm || die "Can't download source package."
 fi
 rpm --checksig kernel-2.4.21-63.EL.src.rpm || die "Can't verify signature."
 rpm -ivh kernel-2.4.21-63.EL.src.rpm || die "Can't install source package."
 
 cd /usr/src/redhat/SOURCES/ || die "Can't chdir to /usr/src/redhat/SOURCES/ ."
-if [ ! -r ccs-patch-1.8.3-20111111.tar.gz ]
+if [ ! -r ccs-patch-1.8.3-20120120.tar.gz ]
 then
-    wget -O ccs-patch-1.8.3-20111111.tar.gz 'http://sourceforge.jp/frs/redir.php?f=/tomoyo/49684/ccs-patch-1.8.3-20111111.tar.gz' || die "Can't download patch."
+    wget -O ccs-patch-1.8.3-20120120.tar.gz 'http://sourceforge.jp/frs/redir.php?f=/tomoyo/49684/ccs-patch-1.8.3-20120120.tar.gz' || die "Can't download patch."
 fi
 
 cd /tmp/ || die "Can't chdir to /tmp/ ."
@@ -33,7 +33,7 @@ patch << "EOF" || die "Can't patch spec file."
  # adding some text to the end of the version number.
  #
 -%define release 63.EL
-+%define release 63.EL_tomoyo_1.8.3p2
++%define release 63.EL_tomoyo_1.8.3p4
  %define sublevel 21
  %define kversion 2.4.%{sublevel}
  # /usr/src/%{kslnk} -> /usr/src/linux-%{KVERREL}
@@ -51,7 +51,7 @@ patch << "EOF" || die "Can't patch spec file."
  # END OF PATCH APPLICATIONS
  
 +# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.8.3-20111111.tar.gz
++tar -zxf %_sourcedir/ccs-patch-1.8.3-20120120.tar.gz
 +patch -sp1 < patches/ccs-patch-2.4.21-centos-3.9.diff
 +
  cp %{SOURCE10} Documentation/
