@@ -1,6 +1,6 @@
 #! /bin/sh
 #
-# This is a kernel build script for Fedora 15's 2.6.41 kernel.
+# This is a kernel build script for Fedora 16's 3.1.0 kernel.
 #
 
 die () {
@@ -8,19 +8,21 @@ die () {
     exit 1
 }
 
+yum -y install wget rpm-build make gcc redhat-rpm-config xmlto asciidoc gnupg elfutils-devel zlib-devel binutils-devel newt-devel python-devel perl-ExtUtils-Embed pciutils-devel
+
 cd /tmp/ || die "Can't chdir to /tmp/ ."
 
-if [ ! -r kernel-2.6.41.4-1.fc15.src.rpm ]
+if [ ! -r kernel-3.1.9-1.fc16.src.rpm ]
 then
-    wget http://ftp.riken.jp/Linux/fedora/updates/15/SRPMS/kernel-2.6.41.4-1.fc15.src.rpm || die "Can't download source package."
+    wget http://ftp.riken.jp/Linux/fedora/updates/16/SRPMS/kernel-3.1.9-1.fc16.src.rpm || die "Can't download source package."
 fi
-rpm --checksig kernel-2.6.41.4-1.fc15.src.rpm || die "Can't verify signature."
-rpm -ivh kernel-2.6.41.4-1.fc15.src.rpm || die "Can't install source package."
+rpm --checksig kernel-3.1.9-1.fc16.src.rpm || die "Can't verify signature."
+rpm -ivh kernel-3.1.9-1.fc16.src.rpm || die "Can't install source package."
 
 cd /root/rpmbuild/SOURCES/ || die "Can't chdir to /root/rpmbuild/SOURCES/ ."
-if [ ! -r ccs-patch-1.8.3-20111213.tar.gz ]
+if [ ! -r ccs-patch-1.8.3-20120120.tar.gz ]
 then
-    wget -O ccs-patch-1.8.3-20111213.tar.gz 'http://sourceforge.jp/frs/redir.php?f=/tomoyo/49684/ccs-patch-1.8.3-20111213.tar.gz' || die "Can't download patch."
+    wget -O ccs-patch-1.8.3-20120120.tar.gz 'http://sourceforge.jp/frs/redir.php?f=/tomoyo/49684/ccs-patch-1.8.3-20120120.tar.gz' || die "Can't download patch."
 fi
 
 cd /root/rpmbuild/SPECS/ || die "Can't chdir to /root/rpmbuild/SPECS/ ."
@@ -28,7 +30,7 @@ cp -p kernel.spec ccs-kernel.spec || die "Can't copy spec file."
 patch << "EOF" || die "Can't patch spec file."
 --- ccs-kernel.spec
 +++ ccs-kernel.spec
-@@ -18,7 +18,7 @@
+@@ -23,7 +23,7 @@
  #
  # (Uncomment the '#' and both spaces below to set the buildid.)
  #
@@ -37,7 +39,7 @@ patch << "EOF" || die "Can't patch spec file."
  ###################################################################
  
  # The buildid can also be specified on the rpmbuild command line
-@@ -389,6 +389,11 @@
+@@ -488,6 +488,11 @@
  # to versions below the minimum
  #
  
@@ -49,7 +51,7 @@ patch << "EOF" || die "Can't patch spec file."
  #
  # First the general kernel 2.6 required versions as per
  # Documentation/Changes
-@@ -448,7 +453,7 @@
+@@ -547,7 +552,7 @@
  AutoProv: yes\
  %{nil}
  
@@ -58,7 +60,7 @@ patch << "EOF" || die "Can't patch spec file."
  Group: System Environment/Kernel
  License: GPLv2
  URL: http://www.kernel.org/
-@@ -809,7 +814,7 @@
+@@ -1014,7 +1019,7 @@
  AutoReqProv: no\
  Requires(pre): /usr/bin/find\
  Requires: perl\
@@ -67,18 +69,18 @@ patch << "EOF" || die "Can't patch spec file."
  This package provides kernel headers and makefiles sufficient to build modules\
  against the %{?2:%{2} }kernel package.\
  %{nil}
-@@ -1272,6 +1277,10 @@
+@@ -1614,6 +1619,10 @@
  
  # END OF PATCH APPLICATIONS
  
 +# TOMOYO Linux
-+tar -zxf %_sourcedir/ccs-patch-1.8.3-20111213.tar.gz
-+patch -sp1 < patches/ccs-patch-2.6.41-fedora-15.diff
++tar -zxf %_sourcedir/ccs-patch-1.8.3-20120120.tar.gz
++patch -sp1 < patches/ccs-patch-3.1.0-fedora-16.diff
 +
  %endif
  
  # Any further pre-build tree manipulations happen here.
-@@ -1300,6 +1309,9 @@
+@@ -1643,6 +1652,9 @@
  for i in *.config
  do
    mv $i .config
