@@ -1,9 +1,9 @@
 /*
  * security/ccsecurity/internal.h
  *
- * Copyright (C) 2005-2011  NTT DATA CORPORATION
+ * Copyright (C) 2005-2012  NTT DATA CORPORATION
  *
- * Version: 1.8.3   2011/09/29
+ * Version: 1.8.3+   2011/11/11
  */
 
 #ifndef _SECURITY_CCSECURITY_INTERNAL_H
@@ -80,6 +80,12 @@ struct path {
 
 #endif
 
+#ifndef __printf
+#define __printf(a,b) __attribute__((format(printf,a,b)))
+#endif
+#ifndef __packed
+#define __packed __attribute__((__packed__))
+#endif
 #ifndef bool
 #define bool _Bool
 #endif
@@ -385,15 +391,27 @@ enum ccs_acl_entry_type_index {
 	CCS_TYPE_PATH_NUMBER_ACL,
 	CCS_TYPE_MKDEV_ACL,
 	CCS_TYPE_MOUNT_ACL,
+#ifdef CONFIG_CCSECURITY_MISC
 	CCS_TYPE_ENV_ACL,
+#endif
+#ifdef CONFIG_CCSECURITY_CAPABILITY
 	CCS_TYPE_CAPABILITY_ACL,
+#endif
+#ifdef CONFIG_CCSECURITY_NETWORK
 	CCS_TYPE_INET_ACL,
 	CCS_TYPE_UNIX_ACL,
+#endif
+#ifdef CONFIG_CCSECURITY_IPC
 	CCS_TYPE_SIGNAL_ACL,
+#endif
+#ifdef CONFIG_CCSECURITY_TASK_EXECUTE_HANDLER
 	CCS_TYPE_AUTO_EXECUTE_HANDLER,
 	CCS_TYPE_DENIED_EXECUTE_HANDLER,
+#endif
+#ifdef CONFIG_CCSECURITY_TASK_DOMAIN_TRANSITION
 	CCS_TYPE_AUTO_TASK_ACL,
 	CCS_TYPE_MANUAL_TASK_ACL,
+#endif
 };
 
 /* Index numbers for "struct ccs_condition". */
@@ -495,17 +513,27 @@ enum ccs_grant_log {
 enum ccs_group_id {
 	CCS_PATH_GROUP,
 	CCS_NUMBER_GROUP,
+#ifdef CONFIG_CCSECURITY_NETWORK
 	CCS_ADDRESS_GROUP,
+#endif
 	CCS_MAX_GROUP
 };
 
 /* Index numbers for category of functionality. */
 enum ccs_mac_category_index {
 	CCS_MAC_CATEGORY_FILE,
+#ifdef CONFIG_CCSECURITY_NETWORK
 	CCS_MAC_CATEGORY_NETWORK,
+#endif
+#ifdef CONFIG_CCSECURITY_MISC
 	CCS_MAC_CATEGORY_MISC,
+#endif
+#ifdef CONFIG_CCSECURITY_IPC
 	CCS_MAC_CATEGORY_IPC,
+#endif
+#ifdef CONFIG_CCSECURITY_CAPABILITY
 	CCS_MAC_CATEGORY_CAPABILITY,
+#endif
 	CCS_MAX_MAC_CATEGORY_INDEX
 };
 
@@ -515,7 +543,9 @@ enum ccs_mac_index {
 	CCS_MAC_FILE_OPEN,
 	CCS_MAC_FILE_CREATE,
 	CCS_MAC_FILE_UNLINK,
+#ifdef CONFIG_CCSECURITY_FILE_GETATTR
 	CCS_MAC_FILE_GETATTR,
+#endif
 	CCS_MAC_FILE_MKDIR,
 	CCS_MAC_FILE_RMDIR,
 	CCS_MAC_FILE_MKFIFO,
@@ -534,29 +564,42 @@ enum ccs_mac_index {
 	CCS_MAC_FILE_MOUNT,
 	CCS_MAC_FILE_UMOUNT,
 	CCS_MAC_FILE_PIVOT_ROOT,
+#ifdef CONFIG_CCSECURITY_NETWORK
 	CCS_MAC_NETWORK_INET_STREAM_BIND,
 	CCS_MAC_NETWORK_INET_STREAM_LISTEN,
 	CCS_MAC_NETWORK_INET_STREAM_CONNECT,
 	CCS_MAC_NETWORK_INET_STREAM_ACCEPT,
 	CCS_MAC_NETWORK_INET_DGRAM_BIND,
 	CCS_MAC_NETWORK_INET_DGRAM_SEND,
+#ifdef CONFIG_CCSECURITY_NETWORK_RECVMSG
 	CCS_MAC_NETWORK_INET_DGRAM_RECV,
+#endif
 	CCS_MAC_NETWORK_INET_RAW_BIND,
 	CCS_MAC_NETWORK_INET_RAW_SEND,
+#ifdef CONFIG_CCSECURITY_NETWORK_RECVMSG
 	CCS_MAC_NETWORK_INET_RAW_RECV,
+#endif
 	CCS_MAC_NETWORK_UNIX_STREAM_BIND,
 	CCS_MAC_NETWORK_UNIX_STREAM_LISTEN,
 	CCS_MAC_NETWORK_UNIX_STREAM_CONNECT,
 	CCS_MAC_NETWORK_UNIX_STREAM_ACCEPT,
 	CCS_MAC_NETWORK_UNIX_DGRAM_BIND,
 	CCS_MAC_NETWORK_UNIX_DGRAM_SEND,
+#ifdef CONFIG_CCSECURITY_NETWORK_RECVMSG
 	CCS_MAC_NETWORK_UNIX_DGRAM_RECV,
+#endif
 	CCS_MAC_NETWORK_UNIX_SEQPACKET_BIND,
 	CCS_MAC_NETWORK_UNIX_SEQPACKET_LISTEN,
 	CCS_MAC_NETWORK_UNIX_SEQPACKET_CONNECT,
 	CCS_MAC_NETWORK_UNIX_SEQPACKET_ACCEPT,
+#endif
+#ifdef CONFIG_CCSECURITY_MISC
 	CCS_MAC_ENVIRON,
+#endif
+#ifdef CONFIG_CCSECURITY_IPC
 	CCS_MAC_SIGNAL,
+#endif
+#ifdef CONFIG_CCSECURITY_CAPABILITY
 	CCS_MAC_CAPABILITY_USE_ROUTE_SOCKET,
 	CCS_MAC_CAPABILITY_USE_PACKET_SOCKET,
 	CCS_MAC_CAPABILITY_SYS_REBOOT,
@@ -567,6 +610,7 @@ enum ccs_mac_index {
 	CCS_MAC_CAPABILITY_USE_KERNEL_MODULE,
 	CCS_MAC_CAPABILITY_SYS_KEXEC_LOAD,
 	CCS_MAC_CAPABILITY_SYS_PTRACE,
+#endif
 	CCS_MAX_MAC_INDEX
 };
 
@@ -604,7 +648,9 @@ enum ccs_network_acl_index {
 	CCS_NETWORK_CONNECT, /* connect() operation. */
 	CCS_NETWORK_ACCEPT,  /* accept() operation. */
 	CCS_NETWORK_SEND,    /* send() operation. */
+#ifdef CONFIG_CCSECURITY_NETWORK_RECVMSG
 	CCS_NETWORK_RECV,    /* recv() operation. */
+#endif
 	CCS_MAX_NETWORK_OPERATION
 };
 
@@ -623,7 +669,9 @@ enum ccs_path_acl_index {
 	CCS_TYPE_WRITE,
 	CCS_TYPE_APPEND,
 	CCS_TYPE_UNLINK,
+#ifdef CONFIG_CCSECURITY_FILE_GETATTR
 	CCS_TYPE_GETATTR,
+#endif
 	CCS_TYPE_RMDIR,
 	CCS_TYPE_TRUNCATE,
 	CCS_TYPE_SYMLINK,
@@ -657,9 +705,13 @@ enum ccs_path_stat_index {
 
 /* Index numbers for entry type. */
 enum ccs_policy_id {
+#ifdef CONFIG_CCSECURITY_PORTRESERVE
 	CCS_ID_RESERVEDPORT,
+#endif
 	CCS_ID_GROUP,
+#ifdef CONFIG_CCSECURITY_NETWORK
 	CCS_ID_ADDRESS_GROUP,
+#endif
 	CCS_ID_PATH_GROUP,
 	CCS_ID_NUMBER_GROUP,
 	CCS_ID_AGGREGATOR,
@@ -692,8 +744,8 @@ enum ccs_pref_index {
 
 /* Index numbers for /proc/ccs/ interfaces. */
 enum ccs_proc_interface_index {
-	CCS_DOMAINPOLICY,
-	CCS_EXCEPTIONPOLICY,
+	CCS_DOMAIN_POLICY,
+	CCS_EXCEPTION_POLICY,
 	CCS_PROCESS_STATUS,
 	CCS_STAT,
 	CCS_AUDIT,
@@ -701,7 +753,9 @@ enum ccs_proc_interface_index {
 	CCS_PROFILE,
 	CCS_QUERY,
 	CCS_MANAGER,
+#ifdef CONFIG_CCSECURITY_TASK_EXECUTE_HANDLER
 	CCS_EXECUTE_HANDLER,
+#endif
 };
 
 /* Index numbers for special mount operations. */
@@ -811,13 +865,13 @@ enum ccs_value_type {
 struct ccs_acl_head {
 	struct list_head list;
 	s8 is_deleted; /* true or false or CCS_GC_IN_PROGRESS */
-} __attribute__((__packed__));
+} __packed;
 
 /* Common header for shared entries. */
 struct ccs_shared_acl_head {
 	struct list_head list;
 	atomic_t users;
-} __attribute__((__packed__));
+} __packed;
 
 /* Common header for individual entries. */
 struct ccs_acl_info {
@@ -825,7 +879,8 @@ struct ccs_acl_info {
 	struct ccs_condition *cond; /* Maybe NULL. */
 	s8 is_deleted; /* true or false or CCS_GC_IN_PROGRESS */
 	u8 type; /* One of values in "enum ccs_acl_entry_type_index". */
-} __attribute__((__packed__));
+	u16 perm;
+} __packed;
 
 /* Structure for holding a word. */
 struct ccs_name_union {
@@ -885,7 +940,7 @@ struct ccs_mini_stat {
 	uid_t uid;
 	gid_t gid;
 	ino_t ino;
-	mode_t mode;
+	umode_t mode;
 	dev_t dev;
 	dev_t rdev;
 };
@@ -1013,6 +1068,7 @@ struct ccs_request_info {
 			 */
 			u8 operation;
 		} path_number;
+#ifdef CONFIG_CCSECURITY_NETWORK
 		struct {
 			const u32 *address; /* Big endian. */
 			u16 port; /* Host endian. */
@@ -1029,17 +1085,24 @@ struct ccs_request_info {
 			/* One of values in "enum ccs_network_acl_index". */
 			u8 operation;
 		} unix_network;
+#endif
+#ifdef CONFIG_CCSECURITY_MISC
 		struct {
 			const struct ccs_path_info *name;
 		} environ;
+#endif
+#ifdef CONFIG_CCSECURITY_CAPABILITY
 		struct {
 			/* One of values in "enum ccs_capability_acl_index". */
 			u8 operation;
 		} capability;
+#endif
+#ifdef CONFIG_CCSECURITY_IPC
 		struct {
 			const char *dest_pattern;
 			int sig;
 		} signal;
+#endif
 		struct {
 			const struct ccs_path_info *type;
 			const struct ccs_path_info *dir;
@@ -1047,9 +1110,11 @@ struct ccs_request_info {
 			unsigned long flags;
 			int need_dev;
 		} mount;
+#ifdef CONFIG_CCSECURITY_TASK_DOMAIN_TRANSITION
 		struct {
 			const struct ccs_path_info *domainname;
 		} task;
+#endif
 	} param;
 	/*
 	 * For updating current->ccs_domain_info at ccs_update_task_domain().
@@ -1214,7 +1279,6 @@ struct ccs_task_acl {
  */
 struct ccs_path_acl {
 	struct ccs_acl_info head; /* type = CCS_TYPE_PATH_ACL */
-	u16 perm; /* Bitmask of values in "enum ccs_path_acl_index". */
 	struct ccs_name_union name;
 };
 
@@ -1223,7 +1287,6 @@ struct ccs_path_acl {
  */
 struct ccs_path2_acl {
 	struct ccs_acl_info head; /* type = CCS_TYPE_PATH2_ACL */
-	u8 perm; /* Bitmask of values in "enum ccs_path2_acl_index". */
 	struct ccs_name_union name1;
 	struct ccs_name_union name2;
 };
@@ -1234,7 +1297,6 @@ struct ccs_path2_acl {
  */
 struct ccs_path_number_acl {
 	struct ccs_acl_info head; /* type = CCS_TYPE_PATH_NUMBER_ACL */
-	u8 perm; /* Bitmask of values in "enum ccs_path_number_acl_index". */
 	struct ccs_name_union name;
 	struct ccs_number_union number;
 };
@@ -1242,7 +1304,6 @@ struct ccs_path_number_acl {
 /* Structure for "file mkblock" and "file mkchar" directive. */
 struct ccs_mkdev_acl {
 	struct ccs_acl_info head; /* type = CCS_TYPE_MKDEV_ACL */
-	u8 perm; /* Bitmask of values in "enum ccs_mkdev_acl_index". */
 	struct ccs_name_union name;
 	struct ccs_number_union mode;
 	struct ccs_number_union major;
@@ -1282,7 +1343,6 @@ struct ccs_signal_acl {
 struct ccs_inet_acl {
 	struct ccs_acl_info head; /* type = CCS_TYPE_INET_ACL */
 	u8 protocol;
-	u8 perm; /* Bitmask of values in "enum ccs_network_acl_index" */
 	struct ccs_ipaddr_union address;
 	struct ccs_number_union port;
 };
@@ -1291,7 +1351,6 @@ struct ccs_inet_acl {
 struct ccs_unix_acl {
 	struct ccs_acl_info head; /* type = CCS_TYPE_UNIX_ACL */
 	u8 protocol;
-	u8 perm; /* Bitmask of values in "enum ccs_network_acl_index" */
 	struct ccs_name_union name;
 };
 
@@ -1308,6 +1367,30 @@ struct ccs_acl_param {
 	struct list_head *list; /* List to add or remove. */
 	struct ccs_policy_namespace *ns; /* Namespace to use. */
 	bool is_delete; /* True if it is a delete request. */
+	union ccs_acl_union {
+		struct ccs_acl_info acl_info;
+		struct ccs_handler_acl handler_acl;
+		struct ccs_task_acl task_acl;
+		struct ccs_path_acl path_acl;
+		struct ccs_path2_acl path2_acl;
+		struct ccs_path_number_acl path_number_acl;
+		struct ccs_mkdev_acl mkdev_acl;
+		struct ccs_mount_acl mount_acl;
+		struct ccs_env_acl env_acl;
+		struct ccs_capability_acl capability_acl;
+		struct ccs_signal_acl signal_acl;
+		struct ccs_inet_acl inet_acl;
+		struct ccs_unix_acl unix_acl;
+		/**/
+		struct ccs_acl_head acl_head;
+		struct ccs_transition_control transition_control;
+		struct ccs_aggregator aggregator;
+		struct ccs_reserved reserved;
+		struct ccs_manager manager;
+		struct ccs_path_group path_group;
+		struct ccs_number_group number_group;
+		struct ccs_address_group address_group;
+	} e;
 };
 
 /* Structure for reading/writing policy via /proc/ccs/ interfaces. */
@@ -1396,152 +1479,47 @@ struct ccs_policy_namespace {
 
 /* Prototype definition for "struct ccsecurity_operations". */
 
-void __init ccs_capability_init(void);
-void __init ccs_domain_init(void);
-void __init ccs_file_init(void);
-void __init ccs_load_builtin_policy(void);
+void __init ccs_permission_init(void);
 void __init ccs_mm_init(void);
-void __init ccs_mount_init(void);
-void __init ccs_network_init(void);
-void __init ccs_policy_io_init(void);
-void __init ccs_signal_init(void);
 
 /* Prototype definition for internal use. */
 
-bool ccs_address_matches_group(const bool is_ipv6, const u32 *address,
-			       const struct ccs_group *group);
-bool ccs_compare_number_union(const unsigned long value,
-			      const struct ccs_number_union *ptr);
-bool ccs_condition(struct ccs_request_info *r,
-		   const struct ccs_condition *cond);
-bool ccs_correct_domain(const unsigned char *domainname);
-bool ccs_correct_path(const char *filename);
-bool ccs_correct_word(const char *string);
-bool ccs_domain_def(const unsigned char *buffer);
-bool ccs_domain_quota_ok(struct ccs_request_info *r);
 bool ccs_dump_page(struct linux_binprm *bprm, unsigned long pos,
 		   struct ccs_page_dump *dump);
 bool ccs_memory_ok(const void *ptr, const unsigned int size);
-bool ccs_number_matches_group(const unsigned long min, const unsigned long max,
-			      const struct ccs_group *group);
-bool ccs_parse_ipaddr_union(struct ccs_acl_param *param,
-			    struct ccs_ipaddr_union *ptr);
-bool ccs_parse_name_union(struct ccs_acl_param *param,
-			  struct ccs_name_union *ptr);
-bool ccs_parse_number_union(struct ccs_acl_param *param,
-			    struct ccs_number_union *ptr);
-bool ccs_path_matches_pattern(const struct ccs_path_info *filename,
-			      const struct ccs_path_info *pattern);
-bool ccs_permstr(const char *string, const char *keyword);
-bool ccs_str_starts(char **src, const char *find);
 char *ccs_encode(const char *str);
 char *ccs_encode2(const char *str, int str_len);
-char *ccs_init_log(struct ccs_request_info *r, int len, const char *fmt,
-		   va_list args);
-char *ccs_read_token(struct ccs_acl_param *param);
-char *ccs_realpath_from_path(struct path *path);
+char *ccs_realpath(struct path *path);
 const char *ccs_get_exe(void);
-const char *ccs_yesno(const unsigned int value);
-const struct ccs_path_info *ccs_compare_name_union
-(const struct ccs_path_info *name, const struct ccs_name_union *ptr);
-const struct ccs_path_info *ccs_get_domainname(struct ccs_acl_param *param);
 const struct ccs_path_info *ccs_get_name(const char *name);
-const struct ccs_path_info *ccs_path_matches_group
-(const struct ccs_path_info *pathname, const struct ccs_group *group);
-int ccs_close_control(struct ccs_io_buffer *head);
-int ccs_env_perm(struct ccs_request_info *r, const char *env);
-int ccs_execute_permission(struct ccs_request_info *r,
-			   const struct ccs_path_info *filename);
-int ccs_get_path(const char *pathname, struct path *path);
+int ccs_audit_log(struct ccs_request_info *r);
+int ccs_check_acl(struct ccs_request_info *r);
 int ccs_init_request_info(struct ccs_request_info *r, const u8 index);
-int ccs_open_control(const u8 type, struct file *file);
-int ccs_parse_ip_address(struct ccs_acl_param *param, u16 *min, u16 *max);
-int ccs_poll_control(struct file *file, poll_table *wait);
-int ccs_poll_log(struct file *file, poll_table *wait);
-int ccs_supervisor(struct ccs_request_info *r, const char *fmt, ...)
-	__attribute__ ((format(printf, 2, 3)));
-int ccs_symlink_path(const char *pathname, struct ccs_path_info *name);
-int ccs_update_domain(struct ccs_acl_info *new_entry, const int size,
-		      struct ccs_acl_param *param,
-		      bool (*check_duplicate) (const struct ccs_acl_info *,
-					       const struct ccs_acl_info *),
-		      bool (*merge_duplicate) (struct ccs_acl_info *,
-					       struct ccs_acl_info *,
-					       const bool));
-int ccs_update_policy(struct ccs_acl_head *new_entry, const int size,
-		      struct ccs_acl_param *param,
-		      bool (*check_duplicate) (const struct ccs_acl_head *,
-					       const struct ccs_acl_head *));
-int ccs_write_aggregator(struct ccs_acl_param *param);
-int ccs_write_capability(struct ccs_acl_param *param);
-int ccs_write_file(struct ccs_acl_param *param);
-int ccs_write_group(struct ccs_acl_param *param, const u8 type);
-int ccs_write_inet_network(struct ccs_acl_param *param);
-int ccs_write_ipc(struct ccs_acl_param *param);
-int ccs_write_misc(struct ccs_acl_param *param);
-int ccs_write_reserved_port(struct ccs_acl_param *param);
-int ccs_write_transition_control(struct ccs_acl_param *param, const u8 type);
-int ccs_write_unix_network(struct ccs_acl_param *param);
-ssize_t ccs_read_control(struct ccs_io_buffer *head, char __user *buffer,
-			 const size_t buffer_len);
-ssize_t ccs_write_control(struct ccs_io_buffer *head,
-			  const char __user *buffer, const size_t buffer_len);
-struct ccs_condition *ccs_get_condition(struct ccs_acl_param *param);
 struct ccs_domain_info *ccs_assign_domain(const char *domainname,
 					  const bool transit);
-struct ccs_domain_info *ccs_find_domain(const char *domainname);
-struct ccs_group *ccs_get_group(struct ccs_acl_param *param, const u8 idx);
-struct ccs_policy_namespace *ccs_assign_namespace(const char *domainname);
-struct ccs_profile *ccs_profile(const u8 profile);
 u8 ccs_get_config(const u8 profile, const u8 index);
-u8 ccs_parse_ulong(unsigned long *result, char **str);
 void *ccs_commit_ok(void *data, const unsigned int size);
-void ccs_check_acl(struct ccs_request_info *r,
-		   bool (*check_entry) (struct ccs_request_info *,
-					const struct ccs_acl_info *));
-void ccs_convert_time(time_t time, struct ccs_time *p);
+void ccs_del_acl(struct list_head *element);
 void ccs_del_condition(struct list_head *element);
 void ccs_fill_path_info(struct ccs_path_info *ptr);
 void ccs_get_attributes(struct ccs_obj_info *obj);
-void ccs_init_policy_namespace(struct ccs_policy_namespace *ns);
-void ccs_normalize_line(unsigned char *buffer);
 void ccs_notify_gc(struct ccs_io_buffer *head, const bool is_register);
-void ccs_print_ip(char *buf, const unsigned int size,
-		  const struct ccs_ipaddr_union *ptr);
-void ccs_print_ulong(char *buffer, const int buffer_len,
-		     const unsigned long value, const u8 type);
-void ccs_put_name_union(struct ccs_name_union *ptr);
-void ccs_put_number_union(struct ccs_number_union *ptr);
-void ccs_read_log(struct ccs_io_buffer *head);
 void ccs_transition_failed(const char *domainname);
-void ccs_update_stat(const u8 index);
 void ccs_warn_oom(const char *function);
 void ccs_write_log(struct ccs_request_info *r, const char *fmt, ...)
-	__attribute__ ((format(printf, 2, 3)));
-void ccs_write_log2(struct ccs_request_info *r, int len, const char *fmt,
-		    va_list args);
+	__printf(2, 3);
 
 /* Variable definition for internal use. */
 
 extern bool ccs_policy_loaded;
-extern const char * const ccs_condition_keyword[CCS_MAX_CONDITION_KEYWORD];
 extern const char * const ccs_dif[CCS_MAX_DOMAIN_INFO_FLAGS];
-extern const char * const ccs_mac_keywords[CCS_MAX_MAC_INDEX
-					   + CCS_MAX_MAC_CATEGORY_INDEX];
-extern const char * const ccs_mode[CCS_CONFIG_MAX_MODE];
-extern const char * const ccs_path_keyword[CCS_MAX_PATH_OPERATION];
-extern const char * const ccs_proto_keyword[CCS_SOCK_MAX];
-extern const char * const ccs_socket_keyword[CCS_MAX_NETWORK_OPERATION];
 extern const u8 ccs_c2mac[CCS_MAX_CAPABILITY_INDEX];
-extern const u8 ccs_index2category[CCS_MAX_MAC_INDEX];
 extern const u8 ccs_pn2mac[CCS_MAX_PATH_NUMBER_OPERATION];
 extern const u8 ccs_pnnn2mac[CCS_MAX_MKDEV_OPERATION];
 extern const u8 ccs_pp2mac[CCS_MAX_PATH2_OPERATION];
 extern struct ccs_domain_info ccs_kernel_domain;
-extern struct ccs_policy_namespace ccs_kernel_namespace;
 extern struct list_head ccs_condition_list;
 extern struct list_head ccs_domain_list;
-extern struct list_head ccs_manager_list;
 extern struct list_head ccs_name_list[CCS_MAX_HASH];
 extern struct list_head ccs_namespace_list;
 extern struct mutex ccs_policy_lock;
@@ -1567,51 +1545,6 @@ static inline bool ccs_pathcmp(const struct ccs_path_info *a,
 	return a->hash != b->hash || strcmp(a->name, b->name);
 }
 
-/**
- * ccs_same_name_union - Check for duplicated "struct ccs_name_union" entry.
- *
- * @a: Pointer to "struct ccs_name_union".
- * @b: Pointer to "struct ccs_name_union".
- *
- * Returns true if @a == @b, false otherwise.
- */
-static inline bool ccs_same_name_union(const struct ccs_name_union *a,
-				       const struct ccs_name_union *b)
-{
-	return a->filename == b->filename && a->group == b->group;
-}
-
-/**
- * ccs_same_number_union - Check for duplicated "struct ccs_number_union" entry.
- *
- * @a: Pointer to "struct ccs_number_union".
- * @b: Pointer to "struct ccs_number_union".
- *
- * Returns true if @a == @b, false otherwise.
- */
-static inline bool ccs_same_number_union(const struct ccs_number_union *a,
-					 const struct ccs_number_union *b)
-{
-	return a->values[0] == b->values[0] && a->values[1] == b->values[1] &&
-		a->group == b->group && a->value_type[0] == b->value_type[0] &&
-		a->value_type[1] == b->value_type[1];
-}
-
-/**
- * ccs_same_ipaddr_union - Check for duplicated "struct ccs_ipaddr_union" entry.
- *
- * @a: Pointer to "struct ccs_ipaddr_union".
- * @b: Pointer to "struct ccs_ipaddr_union".
- *
- * Returns true if @a == @b, false otherwise.
- */
-static inline bool ccs_same_ipaddr_union(const struct ccs_ipaddr_union *a,
-					 const struct ccs_ipaddr_union *b)
-{
-	return !memcmp(a->ip, b->ip, sizeof(a->ip)) && a->group == b->group &&
-		a->is_ipv6 == b->is_ipv6;
-}
-
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 19)
 
 /**
@@ -1725,7 +1658,7 @@ static inline pid_t ccs_sys_getppid(void)
 {
 	pid_t pid;
 	rcu_read_lock();
-	pid = task_tgid_vnr(current->real_parent);
+	pid = task_tgid_vnr(rcu_dereference(current->real_parent));
 	rcu_read_unlock();
 	return pid;
 }
